/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "widget/block_grid.hpp"
#include "memcard/memorycard.hpp"
#include "nana/gui/widgets/widget.hpp"
#include <nana/gui/widgets/label.hpp>
#include <nana/gui/widgets/panel.hpp>
#include <nana/gui/place.hpp>
#include <memory>

namespace duck::widget {
namespace drawerbase::psxmcgrid {
class Trigger : public nana::drawer_trigger {
	class Measurer;
public:
	Trigger();
	~Trigger() noexcept;

private:
	virtual void attached(nana::widget&, nana::paint::graphics&) override;
	virtual void detached() override;

	std::unique_ptr<Measurer> m_measurer;
	nana::widget* m_widget{nullptr};
};
} //namespace drawerbase::psxmcgrid

//class PSXMCGrid : public nana::panel<true> {
class PSXMCGrid : public nana::widget_object<nana::category::widget_tag, drawerbase::psxmcgrid::Trigger> {
public:
	//explicit PSXMCGrid (const std::string& caption="Slot");
	PSXMCGrid() = delete;
	explicit PSXMCGrid (nana::window wd, const std::string& caption="Slot");
	~PSXMCGrid() noexcept = default;

	void bgcolor(const nana::color&);
	void emplace_back (AnimationWithSize&& anim);

private:
	mc::psx::MemoryCard m_mc;
	nana::place m_place;
	nana::panel<true> m_panel;
	BlockGrid m_grid;
	nana::label m_title;
#if !defined(NDEBUG)
	static bool s_dark_bg_colour;
#endif
};

} //namespace duck::widget
