/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "make_qt_animation.hpp"
#include "memcard/block.hpp"
#include "memcard/icon_fetch.hpp"
#include <QPixmap>

namespace duck {
	std::unique_ptr<widget::AnimatedIcon> make_qt_animation (const mc::psx::ConstBlock& block, int width, int height, float fps) {
		using mc::icon_fetch;

		auto retval = std::make_unique<widget::AnimatedIcon>();

		for (const auto& frame : icon_fetch(block, width, height)) {
			QPixmap pix(width, height);
			pix.loadFromData(reinterpret_cast<const uchar*>(frame.data()), frame.size(), "BMP");
			retval->push_back(std::move(pix));
		}

		return retval;
	}
} //namespace duck
