/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "version_info_win.hpp"
#include "app_config.h"
#include "config.h"
#include "git_version.h"
#include <QLabel>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <memory>
#include <QPushButton>

namespace duck {
namespace {
	const constexpr char g_gpl_text[] =
		APP_SHORT_DESC "\n"
		"Copyright (C) " COPYRIGHT_YEAR "  Michele Santullo\n"
		"\n"
		"This program is free software: you can redistribute it and/or modify\n"
		"it under the terms of the GNU General Public License as published by\n"
		"the Free Software Foundation, either version 3 of the License, or\n"
		"(at your option) any later version.\n"
		"\n"
		"This program is distributed in the hope that it will be useful,\n"
		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
		"GNU General Public License for more details.\n"
		"\n"
		"You should have received a copy of the GNU General Public License\n"
		"along with this program.  If not, see <https://www.gnu.org/licenses/>.\n"
	;

	void add_centered_md_label (const std::unique_ptr<QVBoxLayout>& lay, const QString& text) {
		auto label = std::make_unique<QLabel>();
		label->setTextFormat(Qt::TextFormat::MarkdownText);
		label->setText(text);
		lay->addWidget(label.release(), 0, Qt::AlignCenter);
	}
} //unnamed namespace

VersionInfoWin::VersionInfoWin (QWidget* parent) :
	QDialog(parent)
{
	this->setWindowTitle(tr("Version info") + " " + APP_NAME);

	auto main_lay = std::make_unique<QVBoxLayout>();

	add_centered_md_label(main_lay, QString("**") + APP_NAME + "**");
	add_centered_md_label(main_lay, tr("is part of") + " **" + PROJECT_NAME + "** v" + project_ver());

	if constexpr (sizeof(PROJECT_GIT_SHA1) > 1) {
		add_centered_md_label(main_lay, tr("git revision") + " [" + PROJECT_GIT_SHA1 + "](" + PROJECT_REPO_URL + "/commit/" + PROJECT_GIT_SHA1 + ")");
	}
	add_centered_md_label(main_lay, tr("Official source code repository is available here:"));
	add_centered_md_label(main_lay, QString("[") + PROJECT_REPO_URL + "](" + PROJECT_REPO_URL + ")");

	{
		auto txt = std::make_unique<QLabel>();
		txt->setFrameStyle(QFrame::Panel | QFrame::Sunken);
		txt->setText(g_gpl_text);
		main_lay->addWidget(txt.release());
	}

	{
		auto close = std::make_unique<QPushButton>();
		close->setText(tr("&Close"));
		close->setDefault(true);
		connect(close.get(), &QPushButton::clicked, [this](){this->close();});
		main_lay->addWidget(close.release(), 0, Qt::AlignRight);
	}

	setLayout(main_lay.release());
}
} //namespace duck
