/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "animated_icon.hpp"
#include <QSizePolicy>
#include <QPainter>
#include <algorithm>

namespace duck::widget {
namespace {
} //unnamed namespace

AnimatedIcon::AnimatedIcon (QWidget* parent, float fps, bool loop) :
	QWidget(parent),
	m_frame(0),
	m_fps(fps),
	m_loop(loop)
{
	setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}

AnimatedIcon::~AnimatedIcon() noexcept = default;

QSize AnimatedIcon::minimumSizeHint() const {
	return {static_cast<int>(width()), static_cast<int>(height())};
}

QSize AnimatedIcon::sizeHint() const {
	return {static_cast<int>(width()), static_cast<int>(height())};
}

void AnimatedIcon::push_back (QPixmap&& frame) {
	m_frames.push_back(std::move(frame));
}

std::size_t AnimatedIcon::size() const {
	return m_frames.size();
}

unsigned int AnimatedIcon::width() const {
	return (m_frames.empty() ? EmptyIconWidth : m_frames.front().width());
}

unsigned int AnimatedIcon::height() const {
	return (m_frames.empty() ? EmptyIconHeight : m_frames.front().height());
}

void AnimatedIcon::advance_frame() {
	++m_frame;
	if (m_loop)
		m_frame = (not m_frames.empty() ? m_frame % m_frames.size() : 0);
	else
		m_frame = std::min(m_frame, m_frames.size());
	update();
}

void AnimatedIcon::paintEvent (QPaintEvent* event) {
	if (m_frame < m_frames.size()) {
		QPainter painter(this);
		painter.drawPixmap(0, 0, m_frames[m_frame]);
	}
}

} //namespace duck::widget
