/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <QWidget>
#include <QPixmap>
#include <vector>

namespace duck::widget {
class AnimatedIcon : public QWidget {
	Q_OBJECT
public:
	static const constexpr unsigned int EmptyIconWidth = 50;
	static const constexpr unsigned int EmptyIconHeight = EmptyIconWidth;

	AnimatedIcon (AnimatedIcon&&) = default;
	AnimatedIcon (const AnimatedIcon&) = delete;
	explicit AnimatedIcon (QWidget* parent=nullptr, float fps=24.0f, bool loop=true);
	~AnimatedIcon() noexcept;

	AnimatedIcon& operator= (AnimatedIcon&&) = default;

	QSize minimumSizeHint() const override;
	QSize sizeHint() const override;

	void push_back (QPixmap&& frame);
	std::size_t size() const;

	unsigned int width() const;
	unsigned int height() const;

public slots:
	void advance_frame();

protected:
	void paintEvent (QPaintEvent* event) override;

private:
	std::vector<QPixmap> m_frames;
	std::size_t m_frame;
	float m_fps;
	bool m_loop;
};
} //namespace duck::widget
